@@ -38,7 +38,7 @@ class Link(object):
         return ' '.join(data)
 
     def log(self, message, data):
-        if self.is_bytes(data):
+        if not self.is_text(data):
             LOGGER.info("%s : <%s>" % (message, self.byte_to_hex(data)))
         else:
             LOGGER.info("%s : <%s>" % (message, repr(data)))
@@ -141,7 +141,7 @@ class TCPLink(Link):
             if total_data and time.time() - begin > timeout:
                 break
             #if you got no data at all, wait a little longer
-            elif time.time() - begin > timeout * 2:
+            elif time.time() - begin > (timeout * 2):
                 break
             try:
                 data = self.recv_from_socket(size)
@@ -154,9 +154,6 @@ class TCPLink(Link):
                 else:
                     time.sleep(0.1)
             except:
-                # just need to get out of recv form time to time to check if
-                # still alive
-                time.sleep(0.1)
                 pass
         # Try to convert into str
         try:
@@ -211,7 +208,7 @@ class SerialLink(Link):
       - Device name: depending on operating system.
           e.g. /dev/ttyUSB0 on GNU/Linux or COM3 on Windows.'''
     def __init__(self, port, baudrate=19200, bytesize=8, parity='N',
-                                             stopbits=1, timeout=1):
+                 stopbits=1, timeout=1):
         self.port = port
         self.timeout = timeout
         self.baudrate = baudrate
@@ -224,16 +221,17 @@ class SerialLink(Link):
     def url(self):
         '''Make a connection url.'''
         return 'serial:%s:%d:%d%s%d' % (self.port, self.baudrate,
-                                          self.bytesize, self.parity,
-                                          self.stopbits)
+                                        self.bytesize, self.parity,
+                                        self.stopbits)
 
     def open(self):
         '''Open the serial connection.'''
         if self._serial is None:
             self._serial = serial.Serial(self.port, self.baudrate,
-                                    timeout=self.timeout,
-                                    bytesize=self.bytesize, parity=self.parity,
-                                    stopbits=self.stopbits)
+                                         timeout=self.timeout,
+                                         bytesize=self.bytesize,
+                                         parity=self.parity,
+                                         stopbits=self.stopbits)
             LOGGER.info('new %s was initialized' % self)
 
     def settimeout(self, timeout):
@@ -257,7 +255,10 @@ class SerialLink(Link):
     def write(self, data):
         '''Write all `data` to the serial connection.'''
         self.serial.write(data)
-        self.log("Write", data)
+        try:
+            self.log("Write", str(data, encoding='utf8'))
+        except:
+            self.log("Write", data)
 
     def read(self, size=None, timeout=None):
         '''Read data from the serial connection. The maximum amount of data
@@ -267,6 +268,76 @@ class SerialLink(Link):
         timeout = (timeout or 1) * (self.timeout or 1)
         self.serial.timeout = timeout
         data = self.serial.read(size)
+        try:
+            data = str(data, encoding='utf8')
+        except:
+            pass
         self.log("Read", data)
         self.serial.timeout = self.timeout
         return data
+
+
+class GSMLink(Link):
+    '''GSM link class.'''
+
+    def __init__(self, phone, link):
+        self.link = link
+        self.phone = phone
+        self.is_open = False
+
+    def _call(self):
+        LOGGER.info("GSM : Call %s" % self.phone)
+        self.link.write("ATD%s\r\n" % self.phone)
+        while range(100):
+            response = self.link.read(22)
+            if ("BUSY" in response or "NO CARRIER" in response
+                    or "ERROR" in response):
+                LOGGER.error("GSM : <%s>" % repr(response))
+                return False
+            if "CONNECT" in response:
+                self.log("GSM", "Client is ready (%s)" % response)
+                self.link.read()
+                return True
+            self.log("GSM", "call in progress")
+            time.sleep(1)
+        return False
+
+    def _hangup(self):
+        if self.is_open:
+            self.link.write("+++")
+            self.link.read(6)
+            self.link.write("ATH\r\n")
+            if "OK" not in self.link.read(6):
+                return self._hangup()
+            LOGGER.info("GSM : Hang-up")
+
+    def open(self):
+        '''Open the gsm connection.'''
+        if not self.is_open:
+            self.link.open()
+            self.is_open = self._call()
+            if not self.is_open:
+                raise ValueError('no GSM device')
+
+    def close(self):
+        '''Close the gsm connection.'''
+        if self.is_open:
+            self._hangup()
+            self.link.close()
+            self.is_open = False
+
+    @property
+    def url(self):
+        '''Connection url.'''
+        return 'gsm:%s:%s' % (self.phone, str(self.link))
+
+    def settimeout(self, timeout):
+        self.link.settimeout(timeout)
+
+    def write(self, data):
+        '''Write all `data` to the gsm connection.'''
+        self.link.write(data)
+
+    def read(self, size=None, timeout=None):
+        '''Read data from the serial connection.'''
+        return self.link.read(size, timeout)
