@@ -14,21 +14,19 @@ import logging
 from .compat import NullHandler
 
 
-LOGGER = logging.getLogger('pyvpdriver')
+LOGGER = logging.getLogger('pylink')
 LOGGER.addHandler(NullHandler())
 
 
 def active_logger():
     '''Initialize a speaking logger with stream handler (stderr).'''
-    logger = logging.getLogger('pylink')
+    LOGGER = logging.getLogger('pylink')
 
-    logger.setLevel(logging.INFO)
+    LOGGER.setLevel(logging.INFO)
 
     # Default to logging to stderr.
     formatter = logging.Formatter('%(asctime)s %(levelname)s: %(message)s ')
     stream_handler = logging.StreamHandler()
     stream_handler.setFormatter(formatter)
 
-    logger.addHandler(stream_handler)
-
-    return logger
+    LOGGER.addHandler(stream_handler)
